# -*- coding: utf-8 -*-

import six


def force_text(s, encoding='utf-8', errors='strict'):
    """
    Returns a bytestring version of 's', encoded as specified in 'encoding'.

    If strings_only is True, don't convert (some) non-string-like objects.

    Based on the `django.utils.encoding.smart_str' (https://github.com/django/django/blob/master/django/utils/encoding.py)
    """

    if not isinstance(s, six.string_types):
        try:
            return str(s)
        except UnicodeEncodeError:
            if isinstance(s, Exception):
                # An Exception subclass containing non-ASCII data that doesn't
                # know how to print itself properly. We shouldn't raise a
                # further exception.
                return ' '.join([force_text(arg, encoding, errors) for arg in s])
            return unicode(s).encode(encoding, errors)
    elif isinstance(s, six.text_type):
        return s.encode(encoding, errors)
    elif s and encoding != 'utf-8':
        return s.decode('utf-8', errors).encode(encoding, errors)
    else:
        return s



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
try:
    import urlparse
except ImportError:  # Python 3
    from urllib import parse as urlparse
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import mock
import dgis

from tests import MockGetRequest, load_response, skip_if_no_api_key


class ProjectListTest(unittest.TestCase):

    def setUp(self):
        # From the `http://api.2gis.ru/doc/firms/list/project-list/'_
        self.response = load_response('project_list.json')

    def test(self):
        api = dgis.API('1234567890')

        def validate(url):
            parts = urlparse.urlparse(url)
            query = urlparse.parse_qs(parts[4])

            # Assert that there is no more any parameters
            self.assertEqual(len(query), 3)

        validator = MockGetRequest(validate, self.response)

        with mock.patch('requests.get', validator):
            api.project_list()

    @skip_if_no_api_key
    def test_real(self):
        api = dgis.API(os.environ['DGIS_KEY'])
        response = api.project_list()

        self.assertGreater(response['total'], 0)
        self.assertEqual(response['total'], len(response['result']))



import os
import json
import functools
try:
    import unittest2 as unittest
except ImportError:
    import unittest


def skip_if_no_api_key(func):
    wrapper = unittest.skipIf('DGIS_KEY' not in os.environ,
        'Set environment variable DGIS_KEY to test real requests')

    return wrapper(func)


class MockResponse(object):
    """Mock replacement for a response object
    returned by a `requests.get` function"""

    def __init__(self, json):
        self._json = json

    @property
    def json(self):
        return self._json

    @property
    def text(self):
        """This property is used only with a `register_views` parameter,
        so we can freely return a '1' text, which means, that view had been registered"""

        return '1'


class MockGetRequest(object):
    """Mock replacement for a `requests.get` function"""

    def __init__(self, validator, json):
        self.validator = validator
        self.json = json

    def __call__(self, url):
        self.validator(url)

        return MockResponse(self.json)


def load_response(filename):
    try:
        f = open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'responses', filename))
        return json.load(f)
    finally:
        f.close()



# -*- coding: utf-8 _-*-


from dgis.binder import bind_api


__version__ = '0.8.9'


class API(object):
    """2GIS API"""

    def __init__(self, key, host='catalog.api.2gis.ru', version='1.3', register_views=True):
        """

        Parameters::
            key : user API key
            host : base URL for queries
            version : API version for working
            register_views : send information to stats server about a firm profile viewing
        """

        self.key = key
        self.host = host
        self.version = version
        self.register_views = register_views

    """Projects lists

    http://api.2gis.ru/doc/firms/list/project-list/
    """
    project_list = bind_api(
        path='/project/list',
        allowed_param=[],
    )

    """List of the project cities

    http://api.2gis.ru/doc/firms/list/city-list/
    """
    city_list = bind_api(
        path='/city/list',
        allowed_param=['where', 'project_id']
    )

    def rubricator(self, **kwargs):
        """Rubrics search

        http://api.2gis.ru/doc/firms/list/rubricator/
        """

        # `show_children' parameter must be an integer
        kwargs['show_children'] = int(kwargs.pop('show_children', False))

        return self._rubricator(**kwargs)

    _rubricator = bind_api(
        path='/rubricator',
        allowed_param=['where', 'id', 'parent_id', 'show_children', 'sort'],
    )

    def search(self, **kwargs):
        """Firms search

        http://api.2gis.ru/doc/firms/searches/search/
        """

        point = kwargs.pop('point', False)
        if point:
            kwargs['point'] = '%s,%s' % point

        bound = kwargs.pop('bound', False)
        if bound:
            kwargs['bound[point1]'] = bound[0]
            kwargs['bound[point2]'] = bound[1]

        filters = kwargs.pop('filters', False)
        if filters:
            for k, v in filters.items():
                kwargs['filters[%s]' % k] = v

        return self._search(**kwargs)

    _search = bind_api(
        path='/search',
        allowed_param=['what', 'where', 'point', 'radius', 'bound', 'page', 'pagesize', 'sort', 'filters'],
    )

    def search_in_rubric(self, **kwargs):
        """Firms search in rubric

        http://api.2gis.ru/doc/firms/searches/searchinrubric/
        """

        point = kwargs.pop('point', False)
        if point:
            kwargs['point'] = '%s,%s' % point

        bound = kwargs.pop('bound', False)
        if bound:
            kwargs['bound[point1]'] = bound[0]
            kwargs['bound[point2]'] = bound[1]

        filters = kwargs.pop('filters', False)
        if filters:
            for k, v in filters.items():
                kwargs['filters[%s]' % k] = v

        return self._search_in_rubric(**kwargs)

    _search_in_rubric = bind_api(
        path='/searchinrubric',
        allowed_param=['what', 'where', 'point', 'radius', 'bound', 'page', 'pagesize', 'sort', 'filters'],
    )

    """Firm filials

    http://api.2gis.ru/doc/firms/searches/firmsbyfilialid/
    """
    firms_by_filial_id = bind_api(
        path='/firmsByFilialId',
        allowed_param=['firmid', 'page' 'pagesize'],
    )

    """Adverts search

    http://api.2gis.ru/doc/firms/searches/adssearch/
    """
    ads_search = bind_api(
        path='/ads/search',
        allowed_param=['what', 'where', 'format', 'page', 'pagesize'],
    )

    """Firm profile

    http://api.2gis.ru/doc/firms/profiles/profile/
    """
    profile = bind_api(
        path='/profile',
        allowed_param=['id', 'hash'],
        register_views=True,
    )

    def geo_search(self, **kwargs):
        """Geo search

        http://api.2gis.ru/doc/geo/search/
        """

        if 'types' in kwargs:
            kwargs['types'] = ','.join(kwargs['types'])

        bound = kwargs.pop('bound', False)
        if bound:
            kwargs['bound[point1]'] = bound[0]
            kwargs['bound[point2]'] = bound[1]

        return self._geo_search(**kwargs)

    _geo_search = bind_api(
        path='/geo/search',
        allowed_param=['q', 'types', 'radius', 'limit', 'project', 'bound', 'format'],
    )

    """Information about a geo object"""
    geo_get = bind_api(
        path='/geo/search',
        allowed_param=['id', ],
    )



# -*- coding: utf-8 -*-


class DgisError(Exception):
    """2Gis API error"""

    def __init__(self, code, message, error_code):
        self.code = code
        self.message = message
        self.error_code = error_code

    def __str__(self):
        return self.message



try:
    import urlparse
except ImportError:  # Python 3
    from urllib import parse as urlparse
import unittest

import mock
import dgis

from tests import MockGetRequest


class MainTest(unittest.TestCase):

    def setUp(self):
        self.blank_response = {
            'response_code': '200',
        }

    def test(self):
        api = dgis.API('1234567890')

        def validate(url):
            parts = urlparse.urlparse(url)
            query = urlparse.parse_qs(parts[4])

            self.assertEqual(api.version, query['version'][0])
            self.assertEqual(api.key, query['key'][0])
            self.assertEqual(parts[1], api.host)

        validator = MockGetRequest(validate, self.blank_response)

        with mock.patch('requests.get', validator):
            api.project_list()



# -*- coding: utf-8 -*-


try:
    import urlparse
except ImportError:  # Python 3
    from urllib import parse as urlparse
try:
    from urllib import urlencode
except ImportError:  # Python 3
    from urllib.parse import urlencode

import requests

from dgis.exceptions import DgisError
from dgis.utils import force_text

__all__ = ('bind_api',)


def __init__(self, api):
    self.api = api


def execute(self, *args, **kwargs):

    # Build GET parameters for query
    parameters = {}

    # Both positional
    for idx, arg in enumerate(args):
        if arg is None:
            continue

        try:
            parameters[self.allowed_param[idx]] = force_text(arg)
        except IndexError:
            raise ValueError('Too many parameters supplied')

    # And keyword parameters
    for key, arg in kwargs.items():
        if arg is None:
            continue

        if key in parameters:
            raise ValueError('Multiple values for parameter %s supplied' % key)
        parameters[key] = force_text(arg)

    parameters.update({
        'key': self.api.key,
        'version': self.api.version,
        'output': 'json',
    })

    url = urlparse.urlunparse(['http', self.api.host, self.path, None, urlencode(parameters), None])

    response = requests.get(url).json

    if response['response_code'] != '200':
        raise DgisError(int(response['response_code']), response['error_message'], response['error_code'])

    # Register view if required
    if self.register_views and self.api.register_views:
        if requests.get(response['register_bc_url']).text == '0':
            raise DgisError(404, 'View registration cannot be processed', 'registerViewFailed')

    return response


def bind_api(**config):

    properties = {
        'path': config['path'],
        'method': config.get('method', 'GET'),
        'allowed_param': config['allowed_param'],
        'register_views': config.get('register_views', False),
        '__init__': __init__,
        'execute': execute,
    }

    cls = type('API%sMethod' % config['path'].title().replace('/', ''), (object,), properties)

    def _call(api, *args, **kwargs):
        return cls(api).execute(*args, **kwargs)

    return _call



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
try:
    import urlparse
except ImportError:  # Python 3
    from urllib import parse as urlparse
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import mock
import dgis

from tests import MockGetRequest, load_response, skip_if_no_api_key


class RubricatorTest(unittest.TestCase):

    def setUp(self):
        # From the `http://api.2gis.ru/doc/firms/list/rubricator/'_
        self.response = load_response('rubricator.json')

    @unittest.skipIf(False, '')
    def test(self):
        api = dgis.API('1234567890')

        def validate(url):
            parts = urlparse.urlparse(url)
            query = urlparse.parse_qs(parts[4])

            self.assertIn('where', query)

            # Boolean value should be converted into a integer
            self.assertIn('show_children', query)
            self.assertEqual('1', query['show_children'][0])

        validator = MockGetRequest(validate, self.response)

        with mock.patch('requests.get', validator):
            api.rubricator(where='Иркутск', show_children=True)


    @skip_if_no_api_key
    def test_real(self):
        api = dgis.API(os.environ['DGIS_KEY'])
        response = api.rubricator(where='Иркутск', show_children=True)

        self.assertEqual(int(response['total']), len(response['result']))



# -*- coding: utf-8 -*-

"""
python-2gis
============

A Python library for accessing the 2gis API
"""

from setuptools import setup, find_packages


setup(
    name='2gis',
    version='0.8.9',
    author='IRK.ru',
    author_email='support@irk.ru',
    url='https://github.com/irkru/python-2gis',
    description='2gis library for Python',
    long_description=__doc__,
    license='BSD',
    packages=find_packages(),
    install_requires=('requests', 'six'),
    test_suite='tests',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries',
    ),
)
